# Oraperl Emulation Interface for Perl 5 DBD::Oracle DBI
#
# This is a possibly feeble attempt to create an oraperl like
# emulation for mysql
#
# To use this interface use one of the following invocations:
#
#       use Oraperl;
# or
#       eval 'use Oraperl; 1' || die $@ if $] >= 5;
#
# The second form allows oraperl scripts to be used with
# both oraperl and perl 5.

package Myperl;

require 5.002;

use DBI 0.84;
use Exporter;

$VERSION = substr(q$Revision: 1.00 $, 10);

@ISA = qw(Exporter);

@EXPORT = qw(
    &my_login &my_open &my_bind &my_fetch &my_close
    &my_logoff &my_do &my_titles &my_lengths &my_types
    &my_commit &my_rollback &my_autocommit &my_version
    &my_readblob &my_fetchhash
    $my_cache $my_long $my_trunc $my_errno $my_errstr
    $my_verno $my_debug
);

$debug    = 0 unless defined $debug;
$debugdbi = 0;
# $safe		# set true/false before 'use Oraperl' if needed.
$safe = 1 unless defined $safe;

# Help those who get core dumps from non-'safe' Oraperl (bad cursors)
use sigtrap qw(ILL);
if (!$safe) {
    $SIG{BUS} = $SIG{SEGV} = sub {
	print STDERR "Add BEGIN { \$Myperl::safe=1 } above 'use Myperl'.\n"
		unless $safe;
	goto &sigtrap::trap;
    };
}


# Install Driver (use of install_driver is a special case here)
$drh = DBI->install_driver('mysql');
if ($drh) {
    print "DBD::mysql driver installed as $drh\n" if $debug;
    $drh->debug($debug);
    $drh->{CompatMode} = 1;
    $drh->{Warn}       = 0;
}


use strict;

sub _func_ref {
    my $name = shift;
    my $pkg = ($Myperl::safe) ? "DBI" : "DBD::mysql";
    \&{"${pkg}::$name"};
}

sub _warn {
    my $prev_warn = shift;
    if ($_[0] =~ /^(Bad|Duplicate) free/) {
	return unless $ENV{PERL_DBD_DUMP} eq 'dump';
	print STDERR "Aborting with a core dump for diagnostics (PERL_DBD_DUMP)\n";
	dump;
    }
    $prev_warn ? &$prev_warn(@_) : warn @_;
}


#	-----------------------------------------------------------------
#
#	$lda = &ora_login($system_id, $name, $password)
#	&ora_logoff($lda)

sub my_login {
    my($system_id, $name, $password) = @_;
    local($Myperl::prev_warn) = $SIG{'__WARN__'} || 0; # must be local
    local($SIG{'__WARN__'}) = sub { _warn($Myperl::prev_warn, @_) };
    # we still use the old style connect call with an explicit driver
    my $dbh = DBI->connect($system_id, $name, $password, 'mysql');
    return $dbh;
}
sub my_logoff {
    my($dbh) = @_;
    local($Myperl::prev_warn) = $SIG{'__WARN__'} || 0; # must be local
    local($SIG{'__WARN__'}) = sub { _warn($Myperl::prev_warn, @_) };
    $dbh->disconnect();
}



# -----------------------------------------------------------------
#
#   $csr = &ora_open($lda, $stmt [, $cache])
#   &ora_bind($csr, $var, ...)
#   &ora_fetch($csr [, $trunc])
#   &ora_do($lda, $stmt)
#   &ora_close($csr)

sub my_open {
    my($lda, $stmt) = @_;
    $Myperl::ora_cache_o = $_[2];	# temp hack to pass cache through

    my $csr = $lda->prepare($stmt) or return undef;

    # only execute here if no bind vars specified
    $csr->execute or return undef unless $csr->{NUM_OF_PARAMS};

    $csr;
}

sub my_fetch {
    my($csr) = shift;
    return undef if !$csr;
    my(@ret);
    @ret = $csr->fetchrow;
    @ret;
}

sub my_fetchhash 
{
	my($self) = shift;
	my($ref) = $self->fetchrow_hashref;
	if ($ref) {
		return %$ref;
	} else {
		return ();
	}
}

#*my_bind  = _func_ref('st::execute');
#*my_fetch = \&{"DBD::mysql::st::my_fetch"};
#*my_close = _func_ref('st::finish');

sub my_do {
    # error => undef
    # 0     => "0E0"	(0 but true)
    # >0    => >0
    my($lda, $stmt, @params) = @_;	# @params are an extension to the original Oraperl.

    return $lda->do($stmt, undef, @params);	# SEE DEFAULT METHOD IN DBI.pm

    # OLD CODE:
    # $csr is local, cursor will be closed on exit
    my $csr = $lda->prepare($stmt) or return undef;
    # Oracle OCI will automatically execute DDL statements in prepare()!
    # We must be carefull not to execute them again! This needs careful
    # examination and thought.
    # Perhaps oracle is smart enough not to execute them again?
    my $ret = $csr->execute(@params);
    my $rows = $csr->rows;
    ($rows == 0) ? "0E0" : $rows;
}


# -----------------------------------------------------------------
#
#   &ora_titles($csr [, $truncate])
#   &ora_lengths($csr)
#   &ora_types($csr)

sub my_titles{
    my($csr, $trunc) = @_;
    warn "my_titles: truncate option not implemented" if $trunc;
    @{$csr->{'NAME'}};
}

# -----------------------------------------------------------------
#
#   $ora_errno
#   $ora_errstr

# This is really internal knowledge but it saves using tie and
# performance for ora_errno is very important.
*Myperl::my_errno  = \$DBD::mysql::err;
*Myperl::my_errstr = \$DBD::mysql::errstr;


1;
__END__

