#!/usr/bin/perl

# TODO - change all splits to use this
# @service = map { /^={0,1}(.+)/} split /[;\[\]]/, $line;

die "Please specify a hosts.cfg file to import.\n" if ($#ARGV < 0) 

use strict;
use Myperl;

use vars qw($dbh $dbname $dbpass $dbuser $filename %results $line $result);

$filename = $ARGV[0];

die "$filename does not exist!\n" if (!(-f $filename));

$dbname = "netsaint";
$dbuser = "brazil";
$dbpass = "";
$dbh = &my_login("$dbname", "$dbuser","$dbpass") || die "I can't login to database.\n";

open(FL, "<$filename") || die "Cannot open $filename for read.\n";

&delete_table("service");
&delete_table("host");
&delete_table("command");
&delete_table("timeperiod");
&delete_table("time_day");
&delete_table("hostgroup");
&delete_table("hostgroup_contact");
&delete_table("hostgroup_entry");
&delete_table("contact");
&delete_table("contact_host_notify");
&delete_table("contact_service_notify");
&delete_table("contactgroup");
&delete_table("contactgroup_entry");

while (<FL>)
{
	next if (/^#/ || /^\n/ || /^\r/);
	$line = $_;
	chomp($line);
	$results{&process_line($line)}++;
}
close(FL);

foreach $result ( sort keys %results)
{
	printf "%12s - %3d\n", $result, $results{$result};
}

&my_logoff($dbh);

exit(1);

sub delete_table
{
	my $table = shift;
	return if (!defined $table);
	&my_do($dbh, "delete from $table") || die "Can't remove old $table\n$my_errstr\n";
}

sub process_line
{
	my $line = shift;
	my $retcode;
	my $type;

	$line =~ s/^\s+//;
	($type) = split(/\[/, $line);

	$retcode = &load_service($line)      if ($type =~ /^service/);
	$retcode = &load_host($line)         if ($type =~ /^host/);
	$retcode = &load_command($line)      if ($type =~ /^command/);
	$retcode = &load_timeperiod($line)   if ($type =~ /^timeperiod/);
	$retcode = &load_hostgroup($line)    if ($type =~ /^hostgroup/);
	$retcode = &load_contact($line)      if ($type =~ /^contact/);
	$retcode = &load_contactgroup($line) if ($type =~ /^contactgroup/);

	$type = "Failed-$type" if ($retcode == -1);
	return $type;
}

sub load_service
{
	my $line = shift;
	my @service;
	my $csr;
	my $sql;

	@service = split(/[\[\]=;]/, $line);
	($service[15], $service[16]) = split(/\!/, $service[15], 2);

	$service[1] =~ s/\\/\\\\/g;
	$service[1] =~ s/'/''/g;
	$service[3] =~ s/\\/\\\\/g;
	$service[3] =~ s/'/''/g;
	$service[4] =~ s/\\/\\\\/g;
	$service[4] =~ s/'/''/g;
	$service[8] =~ s/\\/\\\\/g;
	$service[8] =~ s/'/''/g;
	$service[10] =~ s/\\/\\\\/g;
	$service[10] =~ s/'/''/g;
	$service[14] =~ s/\\/\\\\/g;
	$service[14] =~ s/'/''/g;
	$service[15] =~ s/\\/\\\\/g;
	$service[15] =~ s/'/''/g;
	$service[16] =~ s/\\/\\\\/g;
	$service[16] =~ s/'/''/g;


	$sql = "insert into service 
		(host, description, check_period, max_attempts, check_interval,
		retry_interval, notification_group, notification_interval,
		notification_period, notify_recovery, notify_critical,
		notify_warning, event_handler, check_command, check_arg)
		values (
		'$service[1]',
		'$service[3]',
		'$service[4]',
		 $service[5],
		 $service[6],
		 $service[7],
		'$service[8]',
		 $service[9],
		'$service[10]',
		 $service[11],
		 $service[12],
		 $service[13],
		'$service[14]',
		'$service[15]',
		'$service[16]')";

	$csr = &my_do($dbh, $sql) || return -1;

	return 1;
}

sub load_host
{
	my $line = shift;
	my @host;
	my $csr;
	my $sql;

	@host = split(/[\[\]=;]/, $line);

	$host[1]  =~ s/'/''/g;
	$host[3]  =~ s/'/''/g;
	$host[4]  =~ s/'/''/g;
	$host[5]  =~ s/'/''/g;
	$host[6]  =~ s/'/''/g;
	$host[9]  =~ s/'/''/g;
	$host[13] =~ s/'/''/g;


	$sql = "insert into host (host_name, host_alias, address, parent_host,
		host_check_command, max_attempts, notification_interval,
		notification_period, notify_recovery, notify_down,
		notify_unreachable, event_handler)
		values (
		'$host[1]',
		'$host[3]',
		'$host[4]',
		'$host[5]',
		'$host[6]',
		 $host[7],
		 $host[8],
		'$host[9]',
		 $host[10],
		 $host[11],
		 $host[12],
		'$host[13]')";

	$csr = &my_do($dbh, $sql) || return -1;

	return 1;

}


sub load_command
{
	my $line = shift;
	my @command;
	my $csr;
	my $sql;

	@command = split(/[\[\]=;]/, $line);
	$command[1] =~ s/\\/\\\\/g;
	$command[1] =~ s/'/''/g;
	$command[3] =~ s/\\/\\\\/g;
	$command[3] =~ s/'/''/g;

	$sql = "insert into command (command_name, command_line) values (
		'$command[1]', '$command[3]')";

	$csr = &my_do($dbh, $sql) || return -1;

	return 1;
}

sub load_timeperiod
{
	my $line = shift;
	my @timeperiod;
	my $csr;
	my $sql;
	my @sun;
	my @mon;
	my @tue;
	my @wed;
	my @thu;
	my @fri;
	my @sat;
	my $start;
	my $stop;

	@timeperiod = split(/[\[\]=;]/, $line);
	
	$timeperiod[1] =~ s/'/''/g;
	$timeperiod[3] =~ s/'/''/g;

	$sql = "insert into timeperiod (timeperiod_name, timeperiod_alias)
		values ('$timeperiod[1]','$timeperiod[3]')";
	$csr = &my_do($dbh, $sql) || return -1;

	@sun = split(/,/, $timeperiod[4]);
	@mon = split(/,/, $timeperiod[5]);
	@tue = split(/,/, $timeperiod[6]);
	@wed = split(/,/, $timeperiod[7]);
	@thu = split(/,/, $timeperiod[8]);
	@fri = split(/,/, $timeperiod[9]);
	@sat = split(/,/, $timeperiod[10]);

	return -1 if (&load_time_day($timeperiod[1], "SUN", @sun) == -1);
	return -1 if (&load_time_day($timeperiod[1], "MON", @mon) == -1);
	return -1 if (&load_time_day($timeperiod[1], "TUE", @tue) == -1);
	return -1 if (&load_time_day($timeperiod[1], "WED", @wed) == -1);
	return -1 if (&load_time_day($timeperiod[1], "THU", @thu) == -1);
	return -1 if (&load_time_day($timeperiod[1], "FRI", @fri) == -1);
	return -1 if (&load_time_day($timeperiod[1], "SAT", @sat) == -1);
	return 1;
}

sub load_time_day
{
	my $tp_name = shift;
	my $day = shift;
	my @times;
	@times = @_;
	my $start;
	my $stop;
	my $i;
	my $sql;
	my $csr;

	for ($i = 0; $i <= $#times; $i++)
	{
		($start, $stop) = split(/-/, $times[$i]);
		$start .= ":00";
		$stop .= ":00";
		$sql = "insert into time_day (timeperiod_name, day,
			start, stop) values (
			'$tp_name','$day','$start','$stop')";

		$csr = &my_do($dbh, $sql) || return -1;
	}
	
	return 1;
}


sub load_hostgroup
{
	my $line = shift;
	my @hostgroup;
	my $csr;
	my $sql;
	my $i;
	my @contacts;
	my @hosts;

	@hostgroup = split(/[\[\]=;]/, $line);

	$hostgroup[1] =~ s/'/''/g;
	$hostgroup[3] =~ s/'/''/g;

	$sql = "insert into hostgroup (group_name, group_alias)
		values ('$hostgroup[1]','$hostgroup[3]')";

	$csr = &my_do($dbh, $sql) || return -1;

	@contacts = split(/,/, $hostgroup[4]);
	@hosts = split(/,/, $hostgroup[5]);

	for ($i = 0; $i <= $#contacts; $i++)
	{
		$sql = "insert into hostgroup_contact (group_name, 
			contact_group) values (
			'$hostgroup[1]','$contacts[$i]')";
		$csr = &my_do($dbh, $sql) || return -1;
	}
	for ($i = 0; $i <= $#hosts; $i++)
	{
		$hosts[$i] =~ s/^ //g;
		$sql = "insert into hostgroup_entry (group_name, 
			host_name) values (
			'$hostgroup[1]','$hosts[$i]')";
		$csr = &my_do($dbh, $sql) || return -1;
	}
	return 1;
}


sub load_contact
{
	my $line = shift;
	my @contact;
	my $csr;
	my $sql;
	my $i;
	my @cmds;

	@contact = split(/[\[\]=;]/, $line);
	for ($i = 0; $i <= $#contact; $i++)
	{
		$contact[$i] =~ s/'/''/g;
	}

	$sql = "insert into contact (contact_name, contact_alias, 
		svc_notification_period, host_notification_period,
		notify_service_recovery, notify_service_critical,
		notify_service_warning, notify_host_recovery,
		notify_host_down, notify_host_unreachable, 
		email_address, pager) values (
		'$contact[1]',
		'$contact[3]',
		'$contact[4]',
		'$contact[5]',
		 $contact[6],
		 $contact[7],
		 $contact[8],
		 $contact[9],
		 $contact[10],
		 $contact[11],
		'$contact[14]',
		'$contact[15]')";

	
#12
#13
	$csr = &my_do($dbh, $sql) || return -1;

	@cmds = split(/,/, $contact[12]);
	for ($i = 0; $i <= $#cmds; $i++)
	{
		$cmds[$i] =~ s/^ //g;
		$sql = "insert into contact_service_notify (contact_name, 
			service_notify_command) values (
			'$contact[1]','$cmds[$i]')";
		$csr = &my_do($dbh, $sql) || return -1;
	}

	@cmds = split(/,/, $contact[13]);
	for ($i = 0; $i <= $#cmds; $i++)
	{
		$cmds[$i] =~ s/^ //g;
		$sql = "insert into contact_host_notify (contact_name, 
			host_notify_command) values (
			'$contact[1]','$cmds[$i]')";
		$csr = &my_do($dbh, $sql) || return -1;
	}


	return 1;
}


sub load_contactgroup
{
	my $line = shift;
	my @group;
	my $csr;
	my $sql;
	my $i;
	my @contacts;

	@group = split(/[\[\]=;]/, $line);
	for ($i = 0; $i <= $#group; $i++)
	{
		$group[$i] =~ s/'/''/g;
	}

	$sql = "insert into contactgroup (group_name, group_alias) values (
		'$group[1]','$group[3]')
		";
	$csr = &my_do($dbh, $sql) || return -1;

	@contacts = split(/,/, $group[4]);
	for ($i = 0; $i <= $#contacts; $i++)
	{
		$sql = "insert into contactgroup_entry (
			group_name, contact_name) values (
			'$group[1]', '$contacts[$i]')";
		$csr = &my_do($dbh, $sql) || return -1;
	}
	return 1;
}


